local function toTimeStamp(date)
   return date:gsub('-', ''):gsub(':', ''):gsub(' ', ''):sub(0, -3)
end   

local function stringToDate(date)
   return date:sub(1,4) .. '-' ..date:sub(5,6) .. '-' .. date:sub(7,8)
end

-- The functions accepts the HTTP params as a parameter.

local AjaxHandlers = {}
function AjaxHandlers.getData(Params)   
   FromDate = Params.from
   ToDate = Params.to
   
   -- Generate a connection handle
   
   conn = db.connect{
      api = db.SQLITE,
      name = 'iguana_monitor',
      live = true
   }
   
   -- Query the data and build time series in JSON format
   
   local query = 'SELECT ' ..
   'RunTime, ' ..
   'InstanceName, ' ..
   'ProcessId, ' ..
   'MemoryMB, ' ..
   'MemRssMB, ' ..
   'DiskUsedPercent, ' ..
   'CpuPercent, ' ..
   'OpenFD, ' ..
   'VersionLabel, ' ..
   'VersionSourceTag, ' ..
   'VersionSourceDateTime, ' ..
   'VersionBuildId ' ..
   'FROM InstanceSummary'
   
   if FromDate or ToDate then
      query = query .. ' WHERE '
      if FromDate then
         FromDate = stringToDate(FromDate)
         query = query .. ' date(RunTime) >= ' .. conn:quote(FromDate)
      else
         query = query .. ' TRUE '
      end
      query = query .. ' AND '
      if ToDate then
         ToDate = stringToDate(ToDate)
         query = query .. ' date(RunTime) <= ' .. conn:quote(ToDate)
      else
         query = query .. ' TRUE '
      end
   end
   
   local Result = conn:query{sql = query, live = true}
   
   local PreviousRunTime = Result[1].RunTime:nodeValue()
   
   local Timeframe = {}
   local Instances = {}
   local Output = {}
   for i = 1, Result:childCount() do
      local Instance = {}
      local CurrentRunTime = Result[i].RunTime:nodeValue()
      
      Instance.name = Result[i].InstanceName:nodeValue()
      Instance.disk = Result[i].DiskUsedPercent:nodeValue()
      Instance.cpu = Result[i].CpuPercent:nodeValue()
      Instance.fd = Result[i].OpenFD:nodeValue()
      Instance.vsize = Result[i].MemoryMB:nodeValue()
      Instance.rsize = Result[i].MemRssMB:nodeValue()
      Instance.pid = Result[i].ProcessId:nodeValue()
      Instance.ver = Result[i].VersionLabel:nodeValue()  ..
      '|' .. Result[i].VersionBuildId:nodeValue() ..
      '|' .. Result[i].VersionSourceTag:nodeValue() ..
      '|' .. Result[i].VersionSourceDateTime:nodeValue()
      
      if PreviousRunTime == CurrentRunTime then
         Instances[#Instances + 1] = Instance
      else
         Timeframe.timestamp = toTimeStamp(PreviousRunTime)
         Timeframe.instances = Instances
         Output[#Output + 1] = Timeframe
         Instances = {}
         Timeframe = {}
         Instances[#Instances + 1] = Instance
         PreviousRunTime = CurrentRunTime
      end
   end
   
   JSON_Output = json.serialize{data = Output}
   
   local Response = net.http.respond{body='var data = ' .. JSON_Output}
end

local function indexPage(Params)
   local HtmlFile = io.open(iguana.project.files()["other/plotter/plot.html"])
   local HtmlContent = HtmlFile:read('*all')
   HtmlFile:close()
   
   net.http.respond{body=HtmlContent}
end

function main(Data)   
   local R = net.http.parseRequest{data=Data}
   
   if AjaxHandlers[R.params.ajax] then
      local Handler = AjaxHandlers[R.params.ajax]
      if iguana.isTest() then
         Handler(R.params)
      else
         local Success, Err = pcall(Handler, R.params)
         if not Success then
            net.http.respond{entity_type='text/json',
               body=json.serialize{data={ErrorMessage=Err}}}
         end
      end
      return
   end
   
   local Handler = indexPage
   if iguana.isTest() then
      Handler(R.params)
   else
      local Success, Err = pcall(Handler, R.params)
      if not Success then
         net.http.respond{body=
            '<html><head><title>Error</title></head><body>'..
            '<div class="err">'..filter.html.enc(Err)..
            '</div></body></html>', code=500}
      end
   end
end

function trace(X) return end